#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import subprocess

from tvm.commands import basecommand
from tvm.shared_helpers.io import info
from tvm.helpers.generatehelper import GenerateHelper


class GenerateCommand(basecommand.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_ignore_errors': True}))

        self.helper = GenerateHelper(self.config, self.collection)
        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        self._path = os.path.abspath(os.path.expanduser(arguments['<path>']))
        self._append = arguments.get('--append', False)

    def _exec(self):
        self.csv_records = self.helper.load_stored_files(self._path)
        if not self.csv_records:
            return info('Uh... No files found in the specified <path>.')

        info('These are the records generated:')
        print(self.csv_records + '\n')
        if sys.platform == 'darwin':
            pipe = subprocess.Popen('pbcopy', stdin=subprocess.PIPE)
            pipe.stdin.write(self.csv_records)
            pipe.stdin.close()

        if not pipe.wait():
            info('Results have been copied to your clipboard.')

        if self._append:
            with open(self.config.added_path, 'a') as shows_recorded:
                shows_recorded.write(self.csv_records)

            info('Results have been appended to the shows_added.csv file.')
