#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from collections import defaultdict

from tvm.helpers import basehelper
from tvm.shared_helpers.io import print_errors
from tvm.shared_helpers.io import err, warn, info, ok


class DoctorHelper(basehelper.BaseHelper):
    def __init__(self, config, collection):
        super(self.__class__, self).__init__(config, collection)

    def _form_key(self, episode_path):
        season_path = os.path.dirname(episode_path)
        show_path = os.path.dirname(season_path)

        episode = os.path.splitext(episode_path)[0]
        return tuple(map(os.path.basename, [season_path, episode, show_path]))

    def _load_stored_files(self):
        stored_shows, errors = {}, defaultdict(list)
        for path in self.config.physical_storage_paths:
            if not os.path.exists(path):
                err('Failed to access path "%s".' % path)
                continue

            ok('Accessing path, "%s".' % path)

            # Walk all defined paths to get all shows stored.
            for root, _, files in os.walk(path):
                for episode in files:
                    episode_path = os.path.join(root, episode)
                    key = self._form_key(episode_path)

                    episode = os.path.splitext(episode_path)[0]
                    if self.config.ignore_pattern.match(os.path.basename(episode)):
                        continue

                    if key in stored_shows:
                        errors['duplicate physical files'].append(episode_path)

                    stored_shows[key] = episode_path

        print_errors(errors, terminate_program=False)
        return stored_shows

    def validate_exists_in_records(self):
        errors = defaultdict(list)
        for stored_show, path in self._load_stored_files().iteritems():
            if stored_show not in self.collection.entries:
                errors['stored and not recorded'].append(path)

        if not errors:
            ok('validate_exists_in_records completed.')
        print_errors(errors)

    def validate_exists_in_repository(self):
        errors = defaultdict(list)
        repository = self._load_stored_files().keys()

        for entry in self.collection.entries.keys():
            if entry not in repository:
                errors['recorded and not stored'].append(entry)

        if not errors:
            ok('validate_exists_in_repository completed.')
        print_errors(errors)
